#pragma once
#include "Scalar.h"

class Vector
{
public:
	Scalar x, y, z;
	/*
	Vector(float x, float y, float z)
	{
		this->x = x;
		this->y = y;
		this->z = z;
	}
	*/
	//Vector() : x(0), y(0), z(0) { }

	Vector(Scalar x = 0, Scalar y = 0, Scalar z = 0) : x(x), y(y), z(z) {}

	void sum(const Vector& v, const Vector& w)
	{
		x = v.x + w.x;
		y = v.y + w.y;
		z = v.z + w.z;
	}

	Vector operator+(const Vector& v) const
	{
		return Vector(x + v.x, y + v.y, z + v.z);
	}

	// unitary minus (flip)
	Vector operator-() const
	{
		return Vector(-x, -y, -z);
	}

	// binary minus (vector difference)
	Vector operator-(const Vector& b) const
	{
		return Vector(x - b.x, y - b.y, z - b.z);
	}

	Vector operator*(Scalar k) const
	{
		return Vector(k * x, k * y, k * z);
	}

	void print() const;

	// declaration and definition done separately
	Vector mix(const Vector& a, const Vector& b, Scalar t);
};

// global stuff...

// commutative closure: k * v is the same as v.operator*(k)
inline Vector operator*(Scalar k, const Vector& v)
{
	return Vector(k * v.x, k * v.y, k * v.z);
}

// use inline instead of implementing in the .cpp file
// used only for short functions

inline Vector Vector::mix(const Vector& a, const Vector& b, Scalar t)
{
	return (1 - t) * a + t * b;
}


Scalar dot(const Vector& a, const Vector& b);

Vector cross(const Vector& a, const Vector& b);

Scalar norm(const Vector&);

Scalar dist(const Vector& a, const Vector& b);
